/*
 *  GOL - Game of life simulation software
 *  Copyright (C) 1996 Michael Raymond Brown
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Author: Michael Raymond Brown
 *  EMail:  michael@arrakeen.demon.co.uk
 *  WWW:    http://www.arrakeen.demon.co.uk/
 *  Address:  Michael Brown
 *            Kia Ora
 *            Brook Street
 *            Great Bromley
 *            Nr Colchester
 *            Essex
 *            CO7 7HX
 *            England
 */


#ifdef __GNUC__
#pragma implementation
#endif

#include <string.h>
#include "Widget.h"
#include "App.h"
#include <Xm/DrawingA.h>


xwHandler::~xwHandler()
   { }

void xwHandler::onMenu(class xwWindow* from, int index)
   { }

void xwHandler::onArm(xwWidget* parent, int index)
   { }
void xwHandler::onDisarm(xwWidget* parent, int index)
   { }

void xwHandler::onClick(xwWidget* parent, int index)
   { }
void xwHandler::onChange(xwWidget* parent, int index)
   { }

void xwHandler::mouseDown(xwWidget* parent, int index, int button, int x, int y)
   { }
void xwHandler::mouseMove(xwWidget* parent, int index, int button, int x, int y)
   { }
void xwHandler::mouseUp(xwWidget* parent, int index, int button, int x, int y)
   { }

void xwHandler::onExpose(xwWidget* parent, int index, int x, int y, int w, int h)
   { }
void xwHandler::onIdle(class xwWidget* from, int index)
   { }
void xwHandler::onDestroy(class xwWindow* from, int index)
   { }

void xwHandler::onResize(class xwWidget* from, int index)
   { }



xwWidget::xwWidget() : widget(0), handler(0), handler_arg(0), button(0), parent(0), parent_app(0)
   {
   }
xwWidget::~xwWidget()
   {if (default_methods & meth_Close)
       {
	destroyWidgets();
	if (widget) XtDestroyWidget(widget);
	if (parent) unlinkParent(parent);
       }
   }

void xwWidget::default_callbacks(unsigned int evnt, unsigned int meth)
   {default_methods = meth;
    default_events  = evnt;

    if (evnt & evnt_Arm)
	XtAddCallback(widget, XmNarmCallback,  cbArm, (XtPointer)this);

    if (evnt & evnt_Disarm)
	XtAddCallback(widget, XmNdisarmCallback,  cbDisarm, (XtPointer)this);

    if (evnt & evnt_Click)
	XtAddCallback(widget, XmNactivateCallback,  cbClick, (XtPointer)this);

    if (evnt & evnt_Change)
	XtAddCallback(widget, XmNvalueChangedCallback, cbChange, (XtPointer)this);

    if (evnt & evnt_Expose)
	XtAddCallback(widget, XmNexposeCallback, cbExpose, (XtPointer)this);

    if (evnt & evnt_Resize)
	XtAddCallback(widget, XmNresizeCallback, cbResize, (XtPointer)this);

    if (evnt & evnt_Motion)
	XtAddEventHandler(widget, Button1MotionMask|Button2MotionMask|Button3MotionMask
				 |ButtonPressMask|ButtonReleaseMask
			, False, cbMotion, (XtPointer)this);
   }


void xwWidget::cbArm(Widget w, XtPointer client_data, XtPointer call_data)
   {xwWidget* me = (xwWidget*)client_data;
    if (me->handler) me->handler->onArm(me, me->handler_arg);
   }
void xwWidget::cbDisarm(Widget w, XtPointer client_data, XtPointer call_data)
   {xwWidget* me = (xwWidget*)client_data;
    if (me->handler) me->handler->onDisarm(me, me->handler_arg);
   }
void xwWidget::cbClick(Widget w, XtPointer client_data, XtPointer call_data)
   {xwWidget* me = (xwWidget*)client_data;
    if (me->handler) me->handler->onClick(me, me->handler_arg);
   }
void xwWidget::cbChange(Widget w, XtPointer client_data, XtPointer call_data)
   {xwWidget* me = (xwWidget*)client_data;
    if (me->handler) me->handler->onChange(me, me->handler_arg);
   }
void xwWidget::cbResize(Widget w, XtPointer client_data, XtPointer call_data)
   {xwWidget* me = (xwWidget*)client_data;
    if (me->handler) me->handler->onResize(me, me->handler_arg);
   }

void xwWidget::cbExpose(Widget w, XtPointer client_data, XtPointer call_data)
   {xwWidget*    me = (xwWidget*)client_data;
    XExposeEvent* e = &(((XmDrawingAreaCallbackStruct*)call_data)->event->xexpose);
    if (me->handler)
	me->handler->onExpose(me, me->handler_arg
		, e->x, e->y, e->width, e->height);
   }

void xwWidget::cbMotion(Widget w, XtPointer client_data, XEvent* ev, Boolean* continue_dispatch)
   {xwWidget*     me = (xwWidget*)client_data;
    if (me->handler && (me->default_events & evnt_Motion)) switch (ev->type)
       {case ButtonPress:
	         if (ev->xbutton.button == Button1) me->button|=xwHandler::button1;
	    else if (ev->xbutton.button == Button2) me->button|=xwHandler::button2;
	    else if (ev->xbutton.button == Button3) me->button|=xwHandler::button3;
	    me->handler->mouseDown(me, me->handler_arg, me->button, ev->xbutton.x, ev->xbutton.y);
	    break;

	case ButtonRelease:
	         if (ev->xbutton.button == Button1) me->button&=~xwHandler::button1;
	    else if (ev->xbutton.button == Button2) me->button&=~xwHandler::button2;
	    else if (ev->xbutton.button == Button3) me->button&=~xwHandler::button3;
	    me->handler->mouseUp(me, me->handler_arg, me->button, ev->xbutton.x, ev->xbutton.y);
	    break;

	case MotionNotify:
	    me->button = ((ev->xmotion.state & Button1Mask)?xwHandler::button1:0)
			|((ev->xmotion.state & Button2Mask)?xwHandler::button2:0)
			|((ev->xmotion.state & Button3Mask)?xwHandler::button3:0);
	    me->handler->mouseMove(me, me->handler_arg, me->button, ev->xmotion.x, ev->xmotion.y);
	    break;
   }   }


void xwWidget::setCaption(const char* data)
   {if (default_methods & meth_Caption)
       {XmString label = XmStringCreateLocalized((char*)data);
	XtVaSetValues(widget, XmNlabelString, label, 0);
	XmStringFree(label);
   }   }
void xwWidget::getCaption(char* data)
   {if (default_methods & meth_Caption)
       {XmString label;
	char*     ptr;
	XtVaGetValues(widget, XmNlabelString, &label, 0);
	XmStringGetLtoR(label, "", &ptr);
	XmStringFree(label);

	strcpy(data, ptr);
	XtFree(ptr);
   }   }

void xwWidget::setValue(int value)
   {if (default_methods & meth_State)
       {unsigned char set = value?1:0;
	XtVaSetValues(widget, XmNset, set, 0);
       }
    else if (default_methods & meth_Value)
       {XtVaSetValues(widget, XmNvalue, value, 0);
   }   }
int xwWidget::getValue()
   {if (default_methods & meth_State)
       {unsigned char set;
	XtVaGetValues(widget, XmNset, &set, 0);
	return set?1:0;
       }
    else if (default_methods & meth_Value)
       {int value;
	XtVaGetValues(widget, XmNvalue, &value, 0);
	return value;
   }   }

void xwWidget::setSize(int width, int height)
   {if (default_methods & meth_Size)
       {XtVaSetValues(widget, XmNwidth, (Dimension)width
			   , XmNheight, (Dimension)height, 0);
   }   }
void xwWidget::getSize(int* width, int* height)
   {if (default_methods & meth_Size)
       {Dimension w, h;
	XtVaGetValues(widget, XmNwidth, &w
			   , XmNheight, &h, 0);
	*width = w; *height = h;
   }   }

void xwWidget::setEnabled(int v)
   {XtVaSetValues(widget, XmNsensitive, (Boolean)v, 0);
   }
int xwWidget::getEnabled()
   {Boolean v;
    XtVaGetValues(widget, XmNsensitive, &v, 0);
    return (int)v;
   }

void xwWidget::move(int x, int y)
   {XtVaSetValues(widget, XmNx, (Position)x, XmNy, (Position)y, 0);
   }


xwApp* xwWidget::getApp()
   {xwWidget* p = this;
    while (!p->parent_app) p=p->parent;
    return (parent_app=p->parent_app);
   }

